/***************************************************************************
                          ksnuffle.h  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999,2000 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef		KSNUFFLE_H
#define		KSNUFFLE_H 
 
#include	<qmessagebox.h>
#include	<qpushbutton.h>

#include	<ktmainwindow.h>
#include	<ksock.h>

#include	"defines.h"
#include	"classes.h"
#include	"qt.h"


class	KNDAbout : public QDialog
{
	Q_OBJECT

	QLabel		lVersion;
	QLabel		lAuthor	;
	QLabel		lEMail	;
	QPushButton	bOK	;
	QWidget		wCaver	;	
	QPixmap		pCaver	;

public:
	KNDAbout (QWidget *);
       ~KNDAbout () ;

protected	:

	virtual	void	paintEvent (QPaintEvent *) ;

protected slots	:

	virtual	void	done (int) ;
}	;


/*  KNDApp								*/
/*  This is the main application class, and represents the main window	*/
/*  of the application.							*/

class	KNDApp : public KTMainWindow 
{
	enum	Showing
	{	ShowMonitors,
		ShowMulti,
		ShowSetup,
		ShowUser
	}	;

 	Q_OBJECT

	KCombo		*monList	;	/* List of monitors	*/
	QLabel		*stateBox	;	/* Monitor state	*/
	QLabel		*frozenBox	;	/* Monitor frozen	*/
	QLabel		*pausedBox	;	/* Monitor paused	*/
	QLabel		*load1Box	;	/* Load 1 value		*/
	QLabel		*load2Box	;	/* Load 1 value		*/
	QLabel		*memBox		;	/* Memory usage box	*/
	QTimer		timer		;	/* Display update timer	*/
	SArray		*iflist		;	/* Interface list	*/

	KNDMulti	*multi		;
	KNDSetup	*setup		;
	KNDUser		*user		;
	KNDDisplay	*display	;
	KNDButBar	*butbar		;
	Showing		showing		;
	long		second		;
	int		keystate	;
	int		memUsed		;

	KConfig		*userconf	;
	char		*prefilter 	;

	QPopupMenu	*monitor_menu	;
	QPopupMenu	*view_menu	;
	QPopupMenu	*help_menu	;

	KNDView		*view		;
	bool		bViewToolbar	;
	bool		bViewStatusbar	;

	KMenuBar::menuPosition menu_bar_pos ;
	KToolBar::BarPosition  tool_bar_pos ;

	QArray<KNDView *> viewlist	;

	void	showView (KNDView *)	;
	void	setView	 (QWidget *)	;
	void		startNsl ()	;

	virtual	void	keyPressEvent	(QKeyEvent *) ;
	virtual	void	keyReleaseEvent	(QKeyEvent *) ;

public:

	KNDApp	() ; 
       ~KNDApp	() ;

	void	initMenuBar		() ;
	void	initToolBar		() ;
	void	initStatusBar		() ;
	void	initView		(KConfig *) ;
	void	saveOptions		(bool) ;
	void	readOptions		() ;
	void	enableCommand		(int id_) ;
	void	disableCommand		(int id_) ;
	void	doExit			(bool) ;

	void	rename			(const char *, const char *) ;
	void	showClient		(int )	;
	void	setState		(KNDView *) ;
	void	setLoad			(KNDView *, int, int, int, int) ;
	void	Finish			(KNDView *) ;

	inline	void	setMemUsed (int u) { memUsed += u ; }

	virtual	bool	queryExit	() ;
	virtual	void	closeEvent	(QCloseEvent *) ;

public slots	:

	void	commandCallback		(int id_) ;
	void	slotMonitorNew		() ;
	void	slotMonitorReplay	() ;
	void	slotMonitorRemote	() ;
	void	slotMonitorClose	() ;
	void	slotMonitorStop		() ;
	void	slotMonitorStart	() ;
	void	slotMonitorFreeze	() ;
	void	slotMonitorResume	() ;
	void	slotMonitorPick		(int) ;
	void	slotMonitorCloseWindow	() ;
	void	slotMonitorSave		() ;
	void	slotMonitorSaveQuit	() ;
	void	slotMonitorQuit		() ;
	void	slotViewToolBar		() ;
	void	slotViewStatusBar	() ;
	void	slotViewMonitors	() ;
	void	slotViewMulti		() ;
	void	slotViewSetup		() ;
	void	slotViewUser		() ;
	void	slotViewConfig		() ;
	void	slotViewTriggers	() ;
	void	slotViewEvents		() ;
	void	slotViewGraphic		() ;
	void	slotViewPackets		() ;
	void	slotHelpContents	() ;
	void	slotHelpAbout		() ;
	void	slotHelpKDE		() ;

	void	clickSet	    	() ;
	void	clickReset	  	() ;
	void	clickClear	  	() ;
	void	clickGo	     		() ;
	void	clickFreeze	 	() ;
	void	clickQuery	 	() ;

	/* The "timerTick" method is called, nominally, once per second	*/
	/* to update displays, etc. The "upToTick" method is called	*/
	/* each time a packet is captured with the packet capture time	*/
	/* (in seconds) to ensure that we definitely keep synchronised.	*/
	/* Otherwise, a packet could appear to have been captured in	*/
	/* the future.							*/
	void	timerTick		() 	;
	long	upToTick		(long)	;

	void	sockRead (KSocket *) ;
}	;   



/*  Extern functions .....						*/
/*									*/

extern	void	Error		(QMessageBox::Icon, CCHAR *, CCHAR *, ...) ;

#endif // KSNUFFLE_H
