/***************************************************************************
                          knd_pcap.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999,2000 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include	"pktinfo.h"
#include	"defines.h"
#include	"classes.h"
#include	"extern.h"

static	PktInfo	*pktFree ;
static	int	cntFree	 ;
static	int	cntAlloc ;
#define	MAXFREE	(DEFMAXPACK*4)

static	PktInfo	emptyInfo ;

/*  decNetInit	: Network decoding initialisation			*/
/*  (returns)	: void		:					*/

void	decNetInit ()
{
	emptyInfo.srce.a = ""	 ;
	emptyInfo.srce.n = ""	 ;
	emptyInfo.srce.p = ""	 ;
	emptyInfo.srce.s = ""	 ;
	emptyInfo.dest.a = ""	 ;
	emptyInfo.dest.n = ""	 ;
	emptyInfo.dest.p = ""	 ;
	emptyInfo.dest.s = ""	 ;
}	

/*  getPktInfo	: Allocate a PktInfo object				*/
/*  (returns)	: PktInfo *	: Allocated object			*/

PktInfo	*getPktInfo ()
{
	PktInfo	*pkt	;

	if (pktFree != 0)
	{
		pkt	  = pktFree	  ;
		pktFree	  = pktFree->next ;
		cntFree  -= 1 ;
	}
	else
	{	pkt	  = new PktInfo	  ;
		cntAlloc += 1 ;
		setMemUsed (sizeof(PktInfo)) ;
	}
#if	0
	fprintf	(stderr, "getPktInfo : %08lx : a=%5d f=%5d\n",
			 (long)pkt, cntAlloc, cntFree) ;
#endif
	/* Initialise the packet information structure by copying from	*/
	/* the empty structure. To minimise time, copy just the stuff	*/
	/* that will _not_ be immediately set by the captured packet.	*/ 
	memcpy	(&pkt      ->datlen,
		 &emptyInfo. datlen,
		 sizeof(PktInfo) - ((UCHAR *)&emptyInfo.datlen - (UCHAR *)&emptyInfo)) ;

	pkt->usecnt  = 1     ;		/* Initial use count		*/
	pkt->decoded = false ;		/* Data won't be decoded yet!	*/

	return	pkt ;
}

/*  freePktInfo	: Free a PktInfo object					*/
/*  pkt		: PktDisp *	: Object to free			*/
/*  (returns)	: void		:					*/

void	freePktInfo
	(	PktInfo	*pkt
	)
{
	if ((pkt->usecnt -= 1) <= 0)
	{
		if (cntFree < MAXFREE)
		{	pkt->next = pktFree ;
			pktFree   = pkt	    ;
			cntFree  += 1	    ;
		}
		else
		{	delete	pkt	    ;
			cntAlloc -= 1	    ;
			setMemUsed (- sizeof(PktInfo)) ;
		}
#if	0
		fprintf	(stderr, "freePktInfo: %08lx : a=%5d f=%5d : freed\n",
				 (long)pkt, cntAlloc, cntFree) ;
#endif
	}
#if	0
	else	fprintf	(stderr, "freePktInfo: %08lx : a=%5d f=%5d : u=%d\n",
				 (long)pkt, cntAlloc, cntFree, pkt->usecnt) ;
#endif
}

/*  usePktInfo	: Increment use on packet information structure		*/
/*  pkt		: PktInfo *	: Packet structure			*/
/*  (returns)	: void		:					*/

void	usePktInfo
	(	PktInfo	*pkt
	)
{
	pkt->usecnt += 1 ;
#if	0
	fprintf	(stderr, "usePktInfo : %08lx : a=%5d f=%5d : u=%d\n",
			 (long)pkt, cntAlloc, cntFree, pkt->usecnt) ;
#endif
}

